@@ -36,11 +36,13 @@
 #include <limits.h>
 
 #include "missing.h"
+#include "sudo_debug.h"
 
 int
 sudo_setgroups(int ngids, const GETGROUPS_T *gids)
 {
     int maxgids, rval;
+    debug_decl(sudo_setgroups, SUDO_DEBUG_UTIL)
 
     rval = setgroups(ngids, gids);
     if (rval == -1 && errno == EINVAL) {
@@ -53,5 +55,5 @@ sudo_setgroups(int ngids, const GETGROUPS_T *gids)
 	if (ngids > maxgids)
 	    rval = setgroups(maxgids, gids);
     }
-    return rval;
+    debug_return_int(rval);
 }
