@@ -64,6 +64,7 @@
 #include "sudoers.h" /* XXX */
 #include "parse.h"
 #include "toke.h"
+#include "gram.h"
 
 /*
  * We must define SIZE_MAX for yacc's skeleton.c.
@@ -82,10 +83,10 @@
  * Globals
  */
 extern int sudolineno;
+extern int last_token;
 extern char *sudoers;
-static int verbose = FALSE;
-int parse_error = FALSE;
-int pedantic = FALSE;
+static bool verbose = false;
+bool parse_error = false;
 int errorlineno = -1;
 char *errorfile = NULL;
 
@@ -104,20 +105,26 @@ static struct member *new_member(char *, int);
 void
 yyerror(const char *s)
 {
+    debug_decl(yyerror, SUDO_DEBUG_PARSER)
+
+    /* If we last saw a newline the error is on the preceding line. */
+    if (last_token == COMMENT)
+	sudolineno--;
+
     /* Save the line the first error occurred on. */
     if (errorlineno == -1) {
-	errorlineno = sudolineno ? sudolineno - 1 : 0;
+	errorlineno = sudolineno;
 	errorfile = estrdup(sudoers);
     }
     if (trace_print != NULL) {
 	LEXTRACE("<*> ");
     } else if (verbose && s != NULL) {
-	warningx(_(">>> %s: %s near line %d <<<"), sudoers, s,
-	    sudolineno ? sudolineno - 1 : 0);
+	warningx(_(">>> %s: %s near line %d <<<"), sudoers, s, sudolineno);
     }
-    parse_error = TRUE;
+    parse_error = true;
+    debug_return;
 }
-#line 110 "gram.y"
+#line 117 "gram.y"
 #ifndef YYSTYPE_DEFINED
 #define YYSTYPE_DEFINED
 typedef union {
@@ -133,7 +140,7 @@ typedef union {
     int tok;
 } YYSTYPE;
 #endif /* YYSTYPE_DEFINED */
-#line 136 "y.tab.c"
+#line 143 "gram.c"
 #define COMMAND 257
 #define ALIAS 258
 #define DEFVAR 259
@@ -631,11 +638,12 @@ short *yyss;
 short *yysslim;
 YYSTYPE *yyvs;
 int yystacksize;
-#line 604 "gram.y"
+#line 611 "gram.y"
 static struct defaults *
 new_default(char *var, char *val, int op)
 {
     struct defaults *d;
+    debug_decl(new_default, SUDO_DEBUG_PARSER)
 
     d = emalloc(sizeof(struct defaults));
     d->var = var;
@@ -646,13 +654,14 @@ new_default(char *var, char *val, int op)
     d->prev = d;
     d->next = NULL;
 
-    return d;
+    debug_return_ptr(d);
 }
 
 static struct member *
 new_member(char *name, int type)
 {
     struct member *m;
+    debug_decl(new_member, SUDO_DEBUG_PARSER)
 
     m = emalloc(sizeof(struct member));
     m->name = name;
@@ -660,7 +669,7 @@ new_member(char *name, int type)
     m->prev = m;
     m->next = NULL;
 
-    return m;
+    debug_return_ptr(m);
 }
 
 /*
@@ -673,6 +682,7 @@ add_defaults(int type, struct member *bmem, struct defaults *defs)
 {
     struct defaults *d;
     struct member_list binding;
+    debug_decl(add_defaults, SUDO_DEBUG_PARSER)
 
     /*
      * We can only call list2tq once on bmem as it will zero
@@ -688,6 +698,8 @@ add_defaults(int type, struct member *bmem, struct defaults *defs)
 	d->binding = binding;
     }
     tq_append(&defaults, defs);
+
+    debug_return;
 }
 
 /*
@@ -698,6 +710,7 @@ static void
 add_userspec(struct member *members, struct privilege *privs)
 {
     struct userspec *u;
+    debug_decl(add_userspec, SUDO_DEBUG_PARSER)
 
     u = emalloc(sizeof(*u));
     list2tq(&u->users, members);
@@ -705,6 +718,8 @@ add_userspec(struct member *members, struct privilege *privs)
     u->prev = u;
     u->next = NULL;
     tq_append(&userspecs, u);
+
+    debug_return;
 }
 
 /*
@@ -720,6 +735,7 @@ init_parser(const char *path, int quiet)
     struct privilege *priv;
     struct cmndspec *cs;
     struct sudo_command *c;
+    debug_decl(init_parser, SUDO_DEBUG_PARSER)
 
     while ((us = tq_pop(&userspecs)) != NULL) {
 	while ((m = tq_pop(&us->users)) != NULL) {
@@ -804,12 +820,14 @@ init_parser(const char *path, int quiet)
     efree(sudoers);
     sudoers = path ? estrdup(path) : NULL;
 
-    parse_error = FALSE;
+    parse_error = false;
     errorlineno = -1;
-    errorfile = NULL;
+    errorfile = sudoers;
     verbose = !quiet;
+
+    debug_return;
 }
-#line 760 "y.tab.c"
+#line 778 "gram.c"
 /* allocate initial stack or double stack size, up to YYMAXDEPTH */
 #if defined(__cplusplus) || defined(__STDC__)
 static int yygrowstack(void)
@@ -1015,127 +1033,127 @@ yyreduce:
     switch (yyn)
     {
 case 1:
-#line 185 "gram.y"
+#line 192 "gram.y"
 { ; }
 break;
 case 5:
-#line 193 "gram.y"
+#line 200 "gram.y"
 {
 			    ;
 			}
 break;
 case 6:
-#line 196 "gram.y"
+#line 203 "gram.y"
 {
 			    yyerrok;
 			}
 break;
 case 7:
-#line 199 "gram.y"
+#line 206 "gram.y"
 {
 			    add_userspec(yyvsp[-1].member, yyvsp[0].privilege);
 			}
 break;
 case 8:
-#line 202 "gram.y"
+#line 209 "gram.y"
 {
 			    ;
 			}
 break;
 case 9:
-#line 205 "gram.y"
+#line 212 "gram.y"
 {
 			    ;
 			}
 break;
 case 10:
-#line 208 "gram.y"
+#line 215 "gram.y"
 {
 			    ;
 			}
 break;
 case 11:
-#line 211 "gram.y"
+#line 218 "gram.y"
 {
 			    ;
 			}
 break;
 case 12:
-#line 214 "gram.y"
+#line 221 "gram.y"
 {
 			    add_defaults(DEFAULTS, NULL, yyvsp[0].defaults);
 			}
 break;
 case 13:
-#line 217 "gram.y"
+#line 224 "gram.y"
 {
 			    add_defaults(DEFAULTS_USER, yyvsp[-1].member, yyvsp[0].defaults);
 			}
 break;
 case 14:
-#line 220 "gram.y"
+#line 227 "gram.y"
 {
 			    add_defaults(DEFAULTS_RUNAS, yyvsp[-1].member, yyvsp[0].defaults);
 			}
 break;
 case 15:
-#line 223 "gram.y"
+#line 230 "gram.y"
 {
 			    add_defaults(DEFAULTS_HOST, yyvsp[-1].member, yyvsp[0].defaults);
 			}
 break;
 case 16:
-#line 226 "gram.y"
+#line 233 "gram.y"
 {
 			    add_defaults(DEFAULTS_CMND, yyvsp[-1].member, yyvsp[0].defaults);
 			}
 break;
 case 18:
-#line 232 "gram.y"
+#line 239 "gram.y"
 {
 			    list_append(yyvsp[-2].defaults, yyvsp[0].defaults);
 			    yyval.defaults = yyvsp[-2].defaults;
 			}
 break;
 case 19:
-#line 238 "gram.y"
+#line 245 "gram.y"
 {
-			    yyval.defaults = new_default(yyvsp[0].string, NULL, TRUE);
+			    yyval.defaults = new_default(yyvsp[0].string, NULL, true);
 			}
 break;
 case 20:
-#line 241 "gram.y"
+#line 248 "gram.y"
 {
-			    yyval.defaults = new_default(yyvsp[0].string, NULL, FALSE);
+			    yyval.defaults = new_default(yyvsp[0].string, NULL, false);
 			}
 break;
 case 21:
-#line 244 "gram.y"
+#line 251 "gram.y"
 {
-			    yyval.defaults = new_default(yyvsp[-2].string, yyvsp[0].string, TRUE);
+			    yyval.defaults = new_default(yyvsp[-2].string, yyvsp[0].string, true);
 			}
 break;
 case 22:
-#line 247 "gram.y"
+#line 254 "gram.y"
 {
 			    yyval.defaults = new_default(yyvsp[-2].string, yyvsp[0].string, '+');
 			}
 break;
 case 23:
-#line 250 "gram.y"
+#line 257 "gram.y"
 {
 			    yyval.defaults = new_default(yyvsp[-2].string, yyvsp[0].string, '-');
 			}
 break;
 case 25:
-#line 256 "gram.y"
+#line 263 "gram.y"
 {
 			    list_append(yyvsp[-2].privilege, yyvsp[0].privilege);
 			    yyval.privilege = yyvsp[-2].privilege;
 			}
 break;
 case 26:
-#line 262 "gram.y"
+#line 269 "gram.y"
 {
 			    struct privilege *p = emalloc(sizeof(*p));
 			    list2tq(&p->hostlist, yyvsp[-2].member);
@@ -1146,51 +1164,51 @@ case 26:
 			}
 break;
 case 27:
-#line 272 "gram.y"
+#line 279 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
-			    yyval.member->negated = FALSE;
+			    yyval.member->negated = false;
 			}
 break;
 case 28:
-#line 276 "gram.y"
+#line 283 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
-			    yyval.member->negated = TRUE;
+			    yyval.member->negated = true;
 			}
 break;
 case 29:
-#line 282 "gram.y"
+#line 289 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, ALIAS);
 			}
 break;
 case 30:
-#line 285 "gram.y"
+#line 292 "gram.y"
 {
 			    yyval.member = new_member(NULL, ALL);
 			}
 break;
 case 31:
-#line 288 "gram.y"
+#line 295 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, NETGROUP);
 			}
 break;
 case 32:
-#line 291 "gram.y"
+#line 298 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, NTWKADDR);
 			}
 break;
 case 33:
-#line 294 "gram.y"
+#line 301 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, WORD);
 			}
 break;
 case 35:
-#line 300 "gram.y"
+#line 307 "gram.y"
 {
 			    list_append(yyvsp[-2].cmndspec, yyvsp[0].cmndspec);
 #ifdef HAVE_SELINUX
@@ -1223,7 +1241,7 @@ case 35:
 			}
 break;
 case 36:
-#line 332 "gram.y"
+#line 339 "gram.y"
 {
 			    struct cmndspec *cs = emalloc(sizeof(*cs));
 			    if (yyvsp[-3].runas != NULL) {
@@ -1250,80 +1268,80 @@ case 36:
 			}
 break;
 case 37:
-#line 358 "gram.y"
+#line 365 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
-			    yyval.member->negated = FALSE;
+			    yyval.member->negated = false;
 			}
 break;
 case 38:
-#line 362 "gram.y"
+#line 369 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
-			    yyval.member->negated = TRUE;
+			    yyval.member->negated = true;
 			}
 break;
 case 39:
-#line 368 "gram.y"
+#line 375 "gram.y"
 {
 			    yyval.string = yyvsp[0].string;
 			}
 break;
 case 40:
-#line 373 "gram.y"
+#line 380 "gram.y"
 {
 			    yyval.string = yyvsp[0].string;
 			}
 break;
 case 41:
-#line 378 "gram.y"
+#line 385 "gram.y"
 {
 			    yyval.seinfo.role = NULL;
 			    yyval.seinfo.type = NULL;
 			}
 break;
 case 42:
-#line 382 "gram.y"
+#line 389 "gram.y"
 {
 			    yyval.seinfo.role = yyvsp[0].string;
 			    yyval.seinfo.type = NULL;
 			}
 break;
 case 43:
-#line 386 "gram.y"
+#line 393 "gram.y"
 {
 			    yyval.seinfo.type = yyvsp[0].string;
 			    yyval.seinfo.role = NULL;
 			}
 break;
 case 44:
-#line 390 "gram.y"
+#line 397 "gram.y"
 {
 			    yyval.seinfo.role = yyvsp[-1].string;
 			    yyval.seinfo.type = yyvsp[0].string;
 			}
 break;
 case 45:
-#line 394 "gram.y"
+#line 401 "gram.y"
 {
 			    yyval.seinfo.type = yyvsp[-1].string;
 			    yyval.seinfo.role = yyvsp[0].string;
 			}
 break;
 case 46:
-#line 400 "gram.y"
+#line 407 "gram.y"
 {
 			    yyval.runas = NULL;
 			}
 break;
 case 47:
-#line 403 "gram.y"
+#line 410 "gram.y"
 {
 			    yyval.runas = yyvsp[-1].runas;
 			}
 break;
 case 48:
-#line 408 "gram.y"
+#line 415 "gram.y"
 {
 			    yyval.runas = emalloc(sizeof(struct runascontainer));
 			    yyval.runas->runasusers = yyvsp[0].member;
@@ -1331,7 +1349,7 @@ case 48:
 			}
 break;
 case 49:
-#line 413 "gram.y"
+#line 420 "gram.y"
 {
 			    yyval.runas = emalloc(sizeof(struct runascontainer));
 			    yyval.runas->runasusers = yyvsp[-2].member;
@@ -1339,7 +1357,7 @@ case 49:
 			}
 break;
 case 50:
-#line 418 "gram.y"
+#line 425 "gram.y"
 {
 			    yyval.runas = emalloc(sizeof(struct runascontainer));
 			    yyval.runas->runasusers = NULL;
@@ -1347,86 +1365,86 @@ case 50:
 			}
 break;
 case 51:
-#line 425 "gram.y"
+#line 432 "gram.y"
 {
 			    yyval.tag.nopasswd = yyval.tag.noexec = yyval.tag.setenv =
 				yyval.tag.log_input = yyval.tag.log_output = UNSPEC;
 			}
 break;
 case 52:
-#line 429 "gram.y"
+#line 436 "gram.y"
 {
-			    yyval.tag.nopasswd = TRUE;
+			    yyval.tag.nopasswd = true;
 			}
 break;
 case 53:
-#line 432 "gram.y"
+#line 439 "gram.y"
 {
-			    yyval.tag.nopasswd = FALSE;
+			    yyval.tag.nopasswd = false;
 			}
 break;
 case 54:
-#line 435 "gram.y"
+#line 442 "gram.y"
 {
-			    yyval.tag.noexec = TRUE;
+			    yyval.tag.noexec = true;
 			}
 break;
 case 55:
-#line 438 "gram.y"
+#line 445 "gram.y"
 {
-			    yyval.tag.noexec = FALSE;
+			    yyval.tag.noexec = false;
 			}
 break;
 case 56:
-#line 441 "gram.y"
+#line 448 "gram.y"
 {
-			    yyval.tag.setenv = TRUE;
+			    yyval.tag.setenv = true;
 			}
 break;
 case 57:
-#line 444 "gram.y"
+#line 451 "gram.y"
 {
-			    yyval.tag.setenv = FALSE;
+			    yyval.tag.setenv = false;
 			}
 break;
 case 58:
-#line 447 "gram.y"
+#line 454 "gram.y"
 {
-			    yyval.tag.log_input = TRUE;
+			    yyval.tag.log_input = true;
 			}
 break;
 case 59:
-#line 450 "gram.y"
+#line 457 "gram.y"
 {
-			    yyval.tag.log_input = FALSE;
+			    yyval.tag.log_input = false;
 			}
 break;
 case 60:
-#line 453 "gram.y"
+#line 460 "gram.y"
 {
-			    yyval.tag.log_output = TRUE;
+			    yyval.tag.log_output = true;
 			}
 break;
 case 61:
-#line 456 "gram.y"
+#line 463 "gram.y"
 {
-			    yyval.tag.log_output = FALSE;
+			    yyval.tag.log_output = false;
 			}
 break;
 case 62:
-#line 461 "gram.y"
+#line 468 "gram.y"
 {
 			    yyval.member = new_member(NULL, ALL);
 			}
 break;
 case 63:
-#line 464 "gram.y"
+#line 471 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, ALIAS);
 			}
 break;
 case 64:
-#line 467 "gram.y"
+#line 474 "gram.y"
 {
 			    struct sudo_command *c = emalloc(sizeof(*c));
 			    c->cmnd = yyvsp[0].command.cmnd;
@@ -1435,7 +1453,7 @@ case 64:
 			}
 break;
 case 67:
-#line 479 "gram.y"
+#line 486 "gram.y"
 {
 			    char *s;
 			    if ((s = alias_add(yyvsp[-2].string, HOSTALIAS, yyvsp[0].member)) != NULL) {
@@ -1445,14 +1463,14 @@ case 67:
 			}
 break;
 case 69:
-#line 489 "gram.y"
+#line 496 "gram.y"
 {
 			    list_append(yyvsp[-2].member, yyvsp[0].member);
 			    yyval.member = yyvsp[-2].member;
 			}
 break;
 case 72:
-#line 499 "gram.y"
+#line 506 "gram.y"
 {
 			    char *s;
 			    if ((s = alias_add(yyvsp[-2].string, CMNDALIAS, yyvsp[0].member)) != NULL) {
@@ -1462,14 +1480,14 @@ case 72:
 			}
 break;
 case 74:
-#line 509 "gram.y"
+#line 516 "gram.y"
 {
 			    list_append(yyvsp[-2].member, yyvsp[0].member);
 			    yyval.member = yyvsp[-2].member;
 			}
 break;
 case 77:
-#line 519 "gram.y"
+#line 526 "gram.y"
 {
 			    char *s;
 			    if ((s = alias_add(yyvsp[-2].string, RUNASALIAS, yyvsp[0].member)) != NULL) {
@@ -1479,7 +1497,7 @@ case 77:
 			}
 break;
 case 80:
-#line 532 "gram.y"
+#line 539 "gram.y"
 {
 			    char *s;
 			    if ((s = alias_add(yyvsp[-2].string, USERALIAS, yyvsp[0].member)) != NULL) {
@@ -1489,96 +1507,96 @@ case 80:
 			}
 break;
 case 82:
-#line 542 "gram.y"
+#line 549 "gram.y"
 {
 			    list_append(yyvsp[-2].member, yyvsp[0].member);
 			    yyval.member = yyvsp[-2].member;
 			}
 break;
 case 83:
-#line 548 "gram.y"
+#line 555 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
-			    yyval.member->negated = FALSE;
+			    yyval.member->negated = false;
 			}
 break;
 case 84:
-#line 552 "gram.y"
+#line 559 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
-			    yyval.member->negated = TRUE;
+			    yyval.member->negated = true;
 			}
 break;
 case 85:
-#line 558 "gram.y"
+#line 565 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, ALIAS);
 			}
 break;
 case 86:
-#line 561 "gram.y"
+#line 568 "gram.y"
 {
 			    yyval.member = new_member(NULL, ALL);
 			}
 break;
 case 87:
-#line 564 "gram.y"
+#line 571 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, NETGROUP);
 			}
 break;
 case 88:
-#line 567 "gram.y"
+#line 574 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, USERGROUP);
 			}
 break;
 case 89:
-#line 570 "gram.y"
+#line 577 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, WORD);
 			}
 break;
 case 91:
-#line 576 "gram.y"
+#line 583 "gram.y"
 {
 			    list_append(yyvsp[-2].member, yyvsp[0].member);
 			    yyval.member = yyvsp[-2].member;
 			}
 break;
 case 92:
-#line 582 "gram.y"
+#line 589 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
-			    yyval.member->negated = FALSE;
+			    yyval.member->negated = false;
 			}
 break;
 case 93:
-#line 586 "gram.y"
+#line 593 "gram.y"
 {
 			    yyval.member = yyvsp[0].member;
-			    yyval.member->negated = TRUE;
+			    yyval.member->negated = true;
 			}
 break;
 case 94:
-#line 592 "gram.y"
+#line 599 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, ALIAS);
 			}
 break;
 case 95:
-#line 595 "gram.y"
+#line 602 "gram.y"
 {
 			    yyval.member = new_member(NULL, ALL);
 			}
 break;
 case 96:
-#line 598 "gram.y"
+#line 605 "gram.y"
 {
 			    yyval.member = new_member(yyvsp[0].string, WORD);
 			}
 break;
-#line 1529 "y.tab.c"
+#line 1547 "gram.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
