@@ -47,9 +47,14 @@
 
 #include "sudo.h"
 #include "sudo_plugin.h"
+#include "sudo_plugin_int.h"
 
 extern int tgetpass_flags; /* XXX */
 
+#if defined(HAVE_DLOPEN) || defined(HAVE_SHL_LOAD)
+sudo_conv_t sudo_conv;	/* NULL in sudo front-end */
+#endif
+
 /*
  * Sudo conversation function.
  */
@@ -91,6 +96,10 @@ sudo_conversation(int num_msgs, const struct sudo_conv_message msgs[],
 		if (msg->msg)
 		    (void) fputs(msg->msg, stderr);
 		break;
+	    case SUDO_CONV_DEBUG_MSG:
+		if (msg->msg)
+		    sudo_debug_write(msg->msg, strlen(msg->msg));
+		break;
 	    default:
 		goto err;
 	}
