@@ -48,15 +48,6 @@ struct io_plugin_1_0 {
 /*
  * Sudo plugin internals.
  */
-
-struct plugin_info {
-    struct plugin_info *prev; /* required */
-    struct plugin_info *next; /* required */
-    const char *path;
-    const char *symbol_name;
-};
-TQ_DECLARE(plugin_info)
-
 struct plugin_container {
     struct plugin_container *prev; /* required */
     struct plugin_container *next; /* required */
@@ -78,8 +69,7 @@ int sudo_conversation(int num_msgs, const struct sudo_conv_message msgs[],
     struct sudo_conv_reply replies[]);
 int _sudo_printf(int msg_type, const char *fmt, ...);
 
-int sudo_load_plugins(const char *conf_file,
-    struct plugin_container *policy_plugin,
+bool sudo_load_plugins(struct plugin_container *policy_plugin,
     struct plugin_container_list *io_plugins);
 
 #endif /* _SUDO_PLUGIN_INT_H */
